@@ -1,7 +1,7 @@
 #########
 # Author:        rmp
-# Last Modified: $Date: 2010-10-29 12:45:25 +0100 (Fri, 29 Oct 2010) $
-# Id:            $Id: Sass.pm 37 2010-10-29 11:45:25Z zerojinx $
+# Last Modified: $Date: 2010-11-07 20:25:32 +0000 (Sun, 07 Nov 2010) $
+# Id:            $Id: Sass.pm 54 2010-11-07 20:25:32Z zerojinx $
 # Source:        $Source$
 # $HeadURL: https://text-sass.svn.sourceforge.net/svnroot/text-sass/trunk/lib/Text/Sass.pm $
 #
@@ -15,9 +15,10 @@ use warnings;
 use Carp;
 use English qw(-no_match_vars);
 use Text::Sass::Expr;
+use Text::Sass::Functions;
 use Data::Dumper;
 
-our $VERSION = q[0.8];
+our $VERSION = q[0.9];
 our $DEBUG   = 0;
 
 sub new {
@@ -39,7 +40,7 @@ sub css2sass {
   }
 
   my $symbols = {};
-  my $stash   = {};
+  my $stash   = [];
   $self->_parse_css($str, $stash, $symbols);
   return $self->_stash2sass($stash, $symbols);
 }
@@ -56,8 +57,20 @@ sub sass2css {
   my $chain   = [];
   $self->{_sass_indent} = 0;
   $self->_parse_sass($str, $stash, $symbols, $chain);
-#  use Data::Dumper; carp 'STASH ' . Dumper($stash);
-#  use Data::Dumper; carp 'SYMBOLS ' . Dumper($symbols);
+
+  return $self->_stash2css($stash, $symbols);
+}
+
+sub scss2css {
+  my ($self, $str) = @_;
+
+  if(!ref $self) {
+    $self = $self->new;
+  }
+
+  my $symbols = {};
+  my $stash   = [];
+  $self->_parse_css($str, $stash, $symbols);
   return $self->_stash2css($stash, $symbols);
 }
 
@@ -95,7 +108,6 @@ sub _parse_sass {
 
       #########
       # !x = y   variable declarations
-      # BOS: Is this deprecated?
       #
       $line =~ s{^\!(\S+)\s*=\s*(.*?)$}{
         $symbols->{variables}->{$1} = $2;
@@ -104,7 +116,7 @@ sub _parse_sass {
       }smxegi;
 
       #########
-      # $x = y   variable declarations
+      # $x : y   variable declarations
       #
       $line =~ s{^\$(\S+)\s*:\s*(.*?)$}{
         $symbols->{variables}->{$1} = $2;
@@ -298,9 +310,9 @@ sub _parse_sass {
           $DEBUG and carp qq[INDENTED $line CALLING DOWN REMAINING=$remaining ].Dumper($substash);
           $self->_parse_sass($remaining, $substash, $symbols, [@{$chain}]);
           $line = q[];
-        }
-        else {
-          croak 'Illegal ident '.length($indent).' we\'re using '.$self->{_sass_indent} . " ($line)";
+
+        } else {
+          croak qq[Illegal indent @{[length $indent]} we're using @{[$self->{_sass_indent}]}  ($line)];
         }
       }
 
@@ -338,19 +350,101 @@ sub _parse_sass {
   return 1;
 }
 
+sub _css_nestedgroups {
+  my ($self, $str) = @_;
+
+  my $groups   = [];
+  my $groupstr = q[];
+  my $indent   = 0;
+
+  for my $i (0..length $str ) {
+    my $char   = substr $str, $i, 1;
+    $groupstr .= $char;
+
+    if ($char eq '{') {
+      $indent++;
+    }
+
+    if ($char eq '}') {
+      $indent--;
+      if ($indent == 0) {
+        push @{$groups}, $groupstr;
+        $groupstr = q[];
+      }
+    }
+  }
+
+  return $groups;
+}
+
+sub _css_kvs {
+  my ($self, $str) = @_;
+
+  my $groups   = [];
+  my $groupstr = q[];
+  my $indent   = 0;
+
+  for my $i (0..length $str) {
+    my $char = substr $str, $i, 1;
+
+    if ($char eq q[;] and $indent == 0) {
+      push @{$groups}, $groupstr;
+      $groupstr = q[];
+
+    } else {
+      $groupstr .= $char;
+    }
+
+    if ($char eq '{') {
+      $indent++;
+    }
+
+    if ($char eq '}') {
+      $indent--;
+      if ($indent == 0) {
+        push @{$groups}, $groupstr;
+        $groupstr = q[];
+      }
+    }
+  }
+
+  return $groups;
+}
+
 sub _parse_css {
   my ($self, $str, $substash, $symbols) = @_;
   $str =~ s{/[*].*?[*]/}{}smxg;
-  my $groups = [$str =~ /[^\n]+{.*?}/smxg];
+
+  #########
+  # scss definitions
+  #
+  while (my ($def) = ( $str =~ m{^\s*(\$[^\n]*)\;}smx )) {
+    $str =~ s{^\s*\$(\S+)\s*:\s*(.*?)\;}{
+      $symbols->{variables}->{$1} = $2;
+      $DEBUG and carp qq[VARIABLE $1 = $2];
+     q[];
+    }smxegi;
+  }
+
+  my $groups = $self->_css_nestedgroups($str);
 
   for my $g (@{$groups}) {
-    my ($tokens, $block) = $g =~ m/(.*){(.*)}/smxg;
+    my ($tokens, $block) = $g =~ m/([^\{]*)\{(.*)\}/smxg;
     $tokens =~ s/^\s+//smx;
     $tokens =~ s/\s+$//smx;
+    $tokens =~ s/\n\s+/\n/smx;
+    $tokens =~ s/\s+\n/\n/smx;
 
-    $substash->{$tokens} ||= {};
+    if ($tokens =~ /^\s*\@mixin\s+(.*)$/smx) {
+      my $proto       = $1;
+      my ($func)      = $1 =~ /^([^(]+)/smx;
+      $symbols->{mixins}->{$func} = "$proto {\n$block\n}\n";
+      $DEBUG and carp qq[MIXIN $func];
+      next;
+    }
 
-    my $kvs = [split /;/smx, $block];
+    my $kvs       = $self->_css_kvs($block);
+    my $ssubstash = [];
 
     for my $kv (@{$kvs}) {
       $kv =~ s/^\s+//smx;
@@ -360,15 +454,52 @@ sub _parse_css {
         next;
       }
 
+      if ($kv =~ /^\s*\@include\s+(.*?)(?:[(](.*?)[)])?$/xms) {
+        my ($func, $argstr) = ($1, $2);
+        my $mixin_str  = $symbols->{mixins}->{$func};
+
+        my $subsymbols = $symbols; # todo: correct scoping - is better as {%{$symbols}}
+        my $values     = $argstr ? [split /\s*,\s*/smx, $argstr] : [];
+        my ($varstr)   = $mixin_str =~ /^.*?[(](.*?)[)]/smx;
+        my ($proto)    = $mixin_str =~ /^\s*([^\{]*\S)\s*\{/smx;
+        my $vars       = $varstr ? [split /\s*,\s*/smx, $varstr] : [];
+
+        for my $var (@{$vars}) {
+          $var =~ s/^[\!\$]//smx;
+          $subsymbols->{variables}->{$var} = shift @{$values};
+        }
+
+        my $result    = [];
+        $self->_parse_css($mixin_str, $result, $subsymbols);
+        push @{$ssubstash}, @{$result->[0]->{$proto}};
+
+        $DEBUG and carp qq[DYNAMIC MIXIN $func];
+        next;
+      }
+
+      if ($kv =~ /^\s*\@extend\s+(.*?)$/xms) {
+        my ($selector) = ($1, $2);
+        carp q[@extend not yet implemented]; ## no critic (RequireInterpolationOfMetachars)
+        next;
+      }
+
+      if ($kv =~ /\{.*\}/smx) {
+        $self->_parse_css( $kv, $ssubstash, $symbols );
+        next;
+      }
+
       my ($key, $value) = split /:/smx, $kv;
       $key   =~ s/^\s+//smx;
       $key   =~ s/\s+$//smx;
       $value =~ s/^\s+//smx;
       $value =~ s/\s+$//smx;
 
-      $substash->{$tokens}->{$key} = $value;
+      push @{$ssubstash}, { $key => $value };
     }
+
+    push @{$substash}, { $tokens => $ssubstash };
   }
+
   return 1;
 }
 
@@ -394,6 +525,7 @@ sub _stash2css {
 	$str .= sprintf q[ %s: %s], $vk, $stash_line->{$k};
 
       } else {
+
 	for my $attr_line (@{$stash_line->{$k}}) {
 	  for my $attr (sort keys %{$attr_line}) {
 	    my $val = $attr_line->{$attr};
@@ -417,6 +549,7 @@ sub _stash2css {
 	      }
 	      next;
 	    }
+
 	    if(ref $val) {
 	      if($attr) {
 		$attr = sprintf q[ %s], $attr;
@@ -432,6 +565,7 @@ sub _stash2css {
 		$attr =~ s/^\s//smx;
 		$rattr = join q[, ], map { "$k $_" } split /[,\s]+/smx, $attr;
 	      }
+
 	      # TODO: What if both have ,?
 
 	      push @{$delayed}, $self->_stash2css([{$rattr => $val}], $symbols);
@@ -446,6 +580,7 @@ sub _stash2css {
       if($str !~ /[{]\s*[}]/smx) {
 	push @{$groups}, $str;
       }
+
       push @{$groups}, @{$delayed};
       $delayed = [];
     }
@@ -461,13 +596,53 @@ sub _expr {
   $expr =~ s/\!(\S+)/{$vars->{$1}||"\!$1"}/smxeg;
   $expr =~ s/\$(\S+)/{$vars->{$1}||"\$$1"}/smxeg;
 
-  my @parts = split /\s+/smx, $expr;
+  {
+    # Functions
+    my $functions = Text::Sass::Functions->new;
 
-  # BOS: Support for functions
+    # TODO: should have rest, so that url() will work
+
+    while ($expr =~ /^(.*?)((\S+)\s*\(([^\)]+)\)(.*)$)/smx) {
+      my $start  = $1;
+      my $mstr   = $2;
+      my $func   = $3;
+      my $varstr = $4;
+      my $end    = $5;
+
+      #########
+      # We want adjust-hue to work
+      #
+      $func =~ s/\-//gsmx;
+
+      if (!$functions->can($func)) {
+        $start = $self->_expr($stash, $symbols, $start);
+        $end   = $self->_expr($stash, $symbols, $end);
+        $expr  = $start . $mstr . $end;
+        last;
+      }
+
+      #########
+      # TODO: Should support darken(#323, something(4+5, 5))
+      #
+      my @vars = split/\,/smx, $varstr;
+      for my $var (@vars) {
+	$var =~ s/^\s//smx;
+	$var = $self->_expr($stash, $symbols, $var);
+      }
+
+      my $res = $functions->$func(@vars);
+      $expr =~ s/\Q$mstr\E/$res/smx
+    }
+  }
+
+  my @parts = split /\s+/smx, $expr;
 
   Readonly::Scalar my $binary_op_parts => 3;
   if(scalar @parts == $binary_op_parts) {
-    return Text::Sass::Expr->expr(@parts);
+    my $ret = Text::Sass::Expr->expr(@parts);
+    if (defined $ret) {
+      return $ret;
+    }
   }
 
   return $expr;
@@ -477,15 +652,20 @@ sub _stash2sass {
   my ($self, $stash, $symbols) = @_;
   my $groups = [];
 
-  for my $k (keys %{$stash}) {
-    my $str .= "$k\n";
+  # TODO: Write symbols
 
-    for my $attr (sort keys %{$stash->{$k}}) {
-      my $val = $stash->{$k}->{$attr};
-      $str   .= sprintf qq[  %s: %s\n], $attr, $val;
+  for my $stashline (@{$stash}) {
+    for my $k (keys %{$stashline}) {
+      my $str .= "$k\n";
+
+      for my $attrline (@{$stashline->{$k}}){
+        for my $attr (sort keys %{$attrline}) {
+          my $val = $attrline->{$attr};
+          $str   .= sprintf qq[  %s: %s\n], $attr, $val;
+        }
+      }
+      push @{$groups}, $str;
     }
-
-    push @{$groups}, $str;
   }
 
   return join "\n", @{$groups};
@@ -500,7 +680,7 @@ Text::Sass
 
 =head1 VERSION
 
-$LastChangedRevision: 37 $
+$LastChangedRevision: 54 $
 
 =head1 SYNOPSIS
 
@@ -520,6 +700,10 @@ $LastChangedRevision: 37 $
 
   my $sCSS = $oSass->sass2css($sSass);
 
+=head2 scss2css - Translate Scss to CSS
+
+  my $sCSS = $oSass->scss2css($sScss);
+
 =head1 DIAGNOSTICS
 
 =head1 CONFIGURATION AND ENVIRONMENT
@@ -536,12 +720,16 @@ $LastChangedRevision: 37 $
 
 =item English
 
+=item Text::Sass::Expr
+
+=item Text::Sass::Functions
+
+=item Data::Dumper
+
 =back
 
 =head1 INCOMPATIBILITIES
 
-There is no support for colorfunctions. There is no support for extends.
-
 =head1 BUGS AND LIMITATIONS
 
 See README
